% [K, fSQL, hSQL, Tifo, sIFO, bHD] = computeIFO(f)
%   aLIGO-like broadband IFO transfer matrix
%
% Arguments:
% f = frequency vector [Hz]
%
% Return:
% K = amplitude to phase conversion term in 2-photon transfer matrix
% Tifo = 2-photon transfer matrix
% fSQL = frequency at which K = 1 in Hz
% hSQL = sqrt(8 * hbar ./ (m * L^2 * w2)) = strain at SQL
% sIFO = 2-photon signal vector
% bHD = 2-photon readout vector

function [K, fSQL, hSQL, Tifo, sIFO, bHD] = computeIFO(f)

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % input numbers
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % physical constants
  hbar = 1.0546e-34; % J s
  c = 299792458;     % m / s
  
  % IFO parameters
  lambda = 1064e-9;
  Titm = 0.014;   % arm cavity input mirror transmission
  
  P = 800e3 * Titm / 2;          % power on BS (W)
  L = 3995;                      % arm length (m)
  m = 40;                        % test mass (kg)
  w0 = 2 * pi * c / lambda;      % carrier frequency (rad/s)
  gammaArm = Titm * c / (4 * L); % arm cavity pole (rad/s)
  
  Lsr = 50;                      % length of signal recycling cavity
  Tsr = 0.35;
  rSR = sqrt(1 - Tsr);
  
  % frequency vector
  w = 2 * pi * f;
  w2 = w.^2;
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % IFO equations
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % interferometer parameters
  Kifo = 8 * P * w0 ./ (m * L^2 * w2 .* (w2 + gammaArm.^2));
  hSQL = sqrt(8 * hbar ./ (m * L^2 * w2));
  phi = w * Lsr / c + atan(w / gammaArm);
  
  % IFO transfer matrix
  K = Tsr * Kifo ./ abs(1 + exp(2i * phi) * rSR).^2;
  Tifo = make2x2TF(1, 0, -K, 1);
  fSQL = exp(interp1(log(K), log(f), 0));    % IFO RPN-shot noise crossing

  % IFO signal
  sIFO = zeros(2, 1, numel(f));
  sIFO(2, 1, :) = sqrt(2 * K) ./ hSQL;
  
  % IFO homodyne
  bHD = zeros(1, 2, numel(f));
  bHD(1, 2, :) = 1;
  
end
