% [nTotal, nAvg, dnX, dnPhi, fc] = noiseInputFC(f, sqzDB, lengthFC, ...
%   lossFC, lossInj, lossRO, mmSqzFC, mmSqzLO, mmPhi, phiRMS, xRMS)
%
% noise of interferometer with "input filtering" of squeezed light
%
% Arguments:
% f = frequency vector
% sqzDB = dB of squeezing at the source
% lengthFC = length of the filter cavity
% lossFC = round-trip loss in the filter cavity
% lossInj = optical loss from squeezer to IFO, not including lossFC
% lossRO = optical loss between IFO and readout (including QE of PD)
% mmSqzFC = squeezer to filter cavity mode-matching
% mmSqzLO = squeezer to local oscillator (e.g. OMC) mode-matching
% mmPhi = mode-matching phase difference (between t00 and tmm)
% phiRMS = RMS of squeezer phase noise
% xRMS = RMS of filter cavity length
%
% Returns:
% nTotal = total noise
% nAvg = noise without RMS motions
% dnPhi = noise added by phiRMS
% dnX = noise added by xRMS
% fc = filter cavity info (see computeFC)
% ifo = struct with return arguments from computeIFO

function [nTotal, nAvg, dnPhi, dnX, fc, ifo] = noiseInputFC(f, sqzDB, ...
  lengthFC, lossFC, lossInj, lossRO, mmSqzFC, mmSqzLO, mmPhi, phiRMS, xRMS)
 
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Squeezer, Interferometer and Filter Cavity
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % physical constants
  c = 299792458;     % m / s
  lambda = 1064e-9;  % m

  % squeezer setup
  Rsqz = log(db2mag(sqzDB));
  Tsqz = make2x2TF(exp(Rsqz), 0, 0, exp(-Rsqz));
  
  % interferometer math
  [K, fSQL, hSQL, Tifo, sIFO, bHD] = computeIFO(f);

  % FC parameters
  fsrFC = c / (2 * lengthFC);

  % detuning and cavity bandwidth
  eps = 4 / (2 + sqrt(2 + 2 * ...
    sqrt(1 + (4 * pi * fSQL / (fsrFC * lossFC))^4)));
  s1eps = sqrt(1 - eps);

  gammaFC = fSQL / sqrt(s1eps + s1eps^3);     % cavity bandwidth [Hz]
  detuneFC = s1eps * gammaFC;                 % cavity detuning [Hz]

  % input mirror transmission
  TinFC = 4 * pi * gammaFC / fsrFC - lossFC;
  if TinFC < lossFC
    error('IFC: Losses are too high! %.1f ppm max.', 1e6 * gammaFC / fsrFC)
  end
  
  %fprintf(1, 'FC1: wide %.1f Hz, detuned %.1f Hz.  Tin = %.1f ppm, eps = %.4f\n', ...
  %    gammaFC, detuneFC, TinFC * 1e6, eps);
  
  % filter cavity properties
  fc = computeFC(f, fsrFC, detuneFC, TinFC, lossFC);
  
  % fiter cavities with displacement noise
  deltaDetuneFC = fsrFC * (2 * xRMS / lambda);
  
  fcA = computeFC(f, fsrFC, detuneFC + deltaDetuneFC, TinFC, lossFC);
  fcB = computeFC(f, fsrFC, detuneFC - deltaDetuneFC, TinFC, lossFC);
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % System Transfer Matrices
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

  % input and output losses
  tInj = sqrt(1 - lossInj); % as named in technical noise paper
  tRO = sqrt(1 - lossRO);   % as named in technical noise paper
  
  % modify Tifo with readout transmission
  TifoRO = tRO * Tifo;
  
  % mode matching, convert from mmSqzFC and mmSqzLO to tmm and t00
  a0 = sqrt(mmSqzFC);
  c0 = sqrt(mmSqzLO);
  b0 = a0 * c0 + sqrt((1 - a0^2) * (1 - c0^2)) * exp(1i * mmPhi);
  
  t00 = a0 * b0;
  tmm = c0 - t00;

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Noise (numerical)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
  % average noise
  nAvg = noiseBackEnd(f, bHD, TifoRO, Tsqz, fc, t00, tmm, tInj, lossRO);
  
  % noise with filter cavity detuning
  nAvgA = noiseBackEnd(f, bHD, TifoRO, Tsqz, fcA, t00, tmm, tInj, lossRO);
  nAvgB = noiseBackEnd(f, bHD, TifoRO, Tsqz, fcB, t00, tmm, tInj, lossRO);
  dnX = (nAvgA + nAvgB) / 2 - nAvg;

  % squzeer phase noise
  TsqzA = getProdTF(make2x2Rot(phiRMS), Tsqz);
  TsqzB = getProdTF(make2x2Rot(-phiRMS), Tsqz);
  
  nAvgA = noiseBackEnd(f, bHD, TifoRO, TsqzA, fc, t00, tmm, tInj, lossRO);
  nAvgB = noiseBackEnd(f, bHD, TifoRO, TsqzB, fc, t00, tmm, tInj, lossRO);
  dnPhi = (nAvgA + nAvgB) / 2 - nAvg;
  
  % noise total
  nTotal = nAvg + dnX + dnPhi;
  
  % build IFO return struct (K, fSQL, hSQL, Tifo, sIFO, bHD)
  if nargout > 5
    ifo.K = K;
    ifo.fSQL = fSQL;
    ifo.hSQL = hSQL;
    ifo.Tifo = Tifo;
    ifo.sIFO = sIFO;
    ifo.bHD = bHD;
  end
  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % Noise (analytical cross check)
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  
end

%%%%%%%%%%%
function nAvg = noiseBackEnd(f, bHD, Tifo, Tsqz, fc, t00, tmm, tInj, lossRO)

  % compute T1, the transfer of the squeezed field
  Tmm = convert1p2p(tmm * ones(size(f)));
  
  T1a = (t00 * fc.Tfc + Tmm) * tInj;
  T1 = getProdTF(Tifo, T1a, Tsqz);
  
  % losses for vacuum v2 an v3 to enter
  tau_u = (t00 * fc.r_u + tmm) * tInj;
  tau_d = (t00 * fc.r_d + tmm) * tInj;

  tau2 = sqrt((abs(tau_u).^2 + abs(tau_d).^2) / 2);
  lambda2 = sqrt(1 - tau2.^2);
  T2 = getProdTF(Tifo, make2x2TF(lambda2, 0, 0, lambda2));
  
  lambda3 = sqrt(lossRO) * ones(size(f));
  T3 = make2x2TF(lambda3, 0, 0, lambda3);

  % curves
  nAvg = computeNoise(bHD, T1, T2, T3);
end
